@@ -1,5 +1,5 @@
-/* Copyright (C) 2000-2010 Boris Wesslowski */
-/* $Id: modes.c 706 2010-10-06 12:02:36Z bw $ */
+/* Copyright (C) 2000-2013 Boris Wesslowski */
+/* $Id: modes.c 731 2013-05-17 14:15:23Z bw $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -54,13 +54,12 @@ void common_input_loop(int *linenum, int *hitnum, int *errnum, int *oldnum, int
     retval = (fgets(buf, BUFSIZE, opt.inputfd) != NULL);
 #ifdef HAVE_ZLIB
   } else {
-    retval = (gzgets(opt.inputfd, buf, BUFSIZE) != Z_NULL);
+    retval = (gzgets(opt.gzinputfd, buf, BUFSIZE) != Z_NULL);
   }
 #endif
 
   while (retval) {
     *linenum += 1;
-    hit = PARSE_NO_HIT;
     hit = parse_line(buf, *linenum);
     opt.repeated = 0;
     switch (hit) {
@@ -85,7 +84,7 @@ void common_input_loop(int *linenum, int *hitnum, int *errnum, int *oldnum, int
       retval = (fgets(buf, BUFSIZE, opt.inputfd) != NULL);
 #ifdef HAVE_ZLIB
     } else {
-      retval = (gzgets(opt.inputfd, buf, BUFSIZE) != Z_NULL);
+      retval = (gzgets(opt.gzinputfd, buf, BUFSIZE) != Z_NULL);
     }
 #endif
   }
@@ -115,14 +114,12 @@ void mode_summary()
 	fprintf(stderr, _("Opening input file '%s'\n"), input);
 
 #ifdef HAVE_ZLIB
-      opt.inputfd = gzopen(input, "rb");
-#else
-      opt.inputfd = fopen(input, "r");
-#endif
-      if (opt.inputfd == NULL) {
-#ifdef HAVE_ZLIB
+      opt.gzinputfd = gzopen(input, "rb");
+      if (opt.gzinputfd == NULL) {
 	fprintf(stderr, "gzopen %s: %s\n", input, strerror(errno));
 #else
+      opt.inputfd = fopen(input, "r");
+      if (opt.inputfd == NULL) {
 	fprintf(stderr, "fopen %s: %s\n", input, strerror(errno));
 #endif
 	exit(EXIT_FAILURE);
@@ -153,10 +150,10 @@ void mode_summary()
       if (retval == EOF) {
 	perror("fclose");
 #else
-      retval = gzclose(opt.inputfd);
+      retval = gzclose(opt.gzinputfd);
       if (retval != 0) {
 	if (retval != Z_ERRNO) {
-	  fprintf(stderr, "gzclose %s: %s\n", input, gzerror(opt.inputfd, &retval));
+	  fprintf(stderr, "gzclose %s: %s\n", input, gzerror(opt.gzinputfd, &retval));
 	} else {
 	  perror("gzclose");
 	}
@@ -356,7 +353,7 @@ void mode_summary()
     if (opt.verbose)
       fprintf(stderr, _("Resolving\n"));
 
-    retval = adns_init(&adns, adns_if_noenv, 0);
+    retval = adns_init(&adns, adns_if_none, 0);
     if (retval) {
       perror("adns_init");
       exit(EXIT_FAILURE);
@@ -426,11 +423,18 @@ void check_pidfile()
 void mode_rt_response_reread_conf()
 {
   free_exclude_data();
-  if (read_rcfile(opt.rcfile, MAY_NOT_EXIST) == EXIT_SUCCESS) {
+  if (read_rcfile(opt.rcfile, MAY_NOT_EXIST, RCFILE_CF) == EXIT_SUCCESS) {
     syslog(LOG_NOTICE, _("SIGHUP caught, reread configuration file %s"), opt.rcfile);
   } else {
     syslog(LOG_NOTICE, _("SIGHUP caught, unable to reread configuration file %s"), opt.rcfile);
   }
+  if (opt.rcfile_dns[0] != '\0') {
+    if (read_rcfile(opt.rcfile_dns, MAY_NOT_EXIST, RCFILE_DNS) == EXIT_SUCCESS) {
+      syslog(LOG_NOTICE, _("Reread DNS cache file %s"), opt.rcfile_dns);
+    } else {
+      syslog(LOG_NOTICE, _("Unable to reread DNS cache file %s"), opt.rcfile_dns);
+    }
+  }
   signal(SIGHUP, mode_rt_response_reread_conf);
 }
 
@@ -629,7 +633,7 @@ void mode_rt_response()
     prepare_socket();
 #ifdef HAVE_ADNS
     if (opt.resolve) {
-      retval = adns_init(&adns, adns_if_noenv, 0);
+      retval = adns_init(&adns, adns_if_none, 0);
       if (retval) {
 	syslog(LOG_NOTICE, "adns_init: %s", strerror(errno));
 	log_exit(EXIT_FAILURE);
@@ -725,14 +729,12 @@ void mode_show_log_times()
     } else {
       input = file->name;
 #ifdef HAVE_ZLIB
-      opt.inputfd = gzopen(input, "rb");
-#else
-      opt.inputfd = fopen(input, "r");
-#endif
-      if (opt.inputfd == NULL) {
-#ifdef HAVE_ZLIB
+      opt.gzinputfd = gzopen(input, "rb");
+      if (opt.gzinputfd == NULL) {
 	fprintf(stderr, "gzopen %s: %s\n", input, strerror(errno));
 #else
+      opt.inputfd = fopen(input, "r");
+      if (opt.inputfd == NULL) {
 	fprintf(stderr, "fopen %s: %s\n", input, strerror(errno));
 #endif
 	exit(EXIT_FAILURE);
@@ -748,7 +750,7 @@ void mode_show_log_times()
       loop = (fgets(buf, BUFSIZE, opt.inputfd) != NULL);
 #ifdef HAVE_ZLIB
     } else {
-      loop = (gzgets(opt.inputfd, buf, BUFSIZE) != Z_NULL);
+      loop = (gzgets(opt.gzinputfd, buf, BUFSIZE) != Z_NULL);
     }
 #endif
 
@@ -770,7 +772,7 @@ void mode_show_log_times()
 	loop = (fgets(buf, BUFSIZE, opt.inputfd) != NULL);
 #ifdef HAVE_ZLIB
       } else {
-	loop = (gzgets(opt.inputfd, buf, BUFSIZE) != Z_NULL);
+	loop = (gzgets(opt.gzinputfd, buf, BUFSIZE) != Z_NULL);
       }
 #endif
     }
@@ -786,10 +788,10 @@ void mode_show_log_times()
       if (retval == EOF) {
 	perror("fclose");
 #else
-      retval = gzclose(opt.inputfd);
+      retval = gzclose(opt.gzinputfd);
       if (retval != 0) {
 	if (retval != Z_ERRNO) {
-	  fprintf(stderr, "gzclose %s: %s\n", input, gzerror(opt.inputfd, &retval));
+	  fprintf(stderr, "gzclose %s: %s\n", input, gzerror(opt.gzinputfd, &retval));
 	} else {
 	  perror("gzclose");
 	}
